import sys
from pathlib import Path
from typing import Generator

# PyInstaller import
import pip_system_certs.wrapt_requests
import cryptography.hazmat.primitives.kdf.pbkdf2


# Application Path
if getattr(sys, 'frozen', False):
    application_path = sys.executable
    application_path = Path(application_path).parent
else:
    application_path = Path().absolute()


def divide_chunks(list_obj: list, size: int) -> Generator:
    """
    Divide a list into chunks of size

    :param list_obj:
    :param size:
    :return: Generator
    """

    for i in range(0, len(list_obj), size):
        yield list_obj[i:i + size]



import logging
from typing import Optional

import numpy as np
import pandas as pd
import oracledb
from oracledb import ConnectionPool, AsyncConnectionPool

from wcp_library.sql import retry, async_retry

logger = logging.getLogger(__name__)


def _connect_warehouse(username: str, password: str, hostname: str, port: int, database: str, min_connections: int,
                       max_connections: int) -> ConnectionPool:
    """
    Create Warehouse Connection

    :param username: username
    :param password: password
    :param hostname: hostname
    :param port: port
    :param database: database
    :param min_connections:
    :param max_connections:
    :return: session_pool
    """

    dsn = oracledb.makedsn(hostname, port, sid=database)
    session_pool = oracledb.create_pool(
        user=username,
        password=password,
        dsn=dsn,
        min=min_connections,
        max=max_connections,
        increment=1,
    )
    return session_pool


async def _async_connect_warehouse(username: str, password: str, hostname: str, port: int, database: str,
                                   min_connections: int, max_connections: int) -> AsyncConnectionPool:
    """
    Create Warehouse Connection

    :param username: username
    :param password: password
    :param hostname: hostname
    :param port: port
    :param database: database
    :param min_connections:
    :param max_connections:
    :return: session_pool
    """

    dsn = oracledb.makedsn(hostname, port, sid=database)
    session_pool = oracledb.create_pool_async(
        user=username,
        password=password,
        dsn=dsn,
        min=min_connections,
        max=max_connections,
        increment=1
    )
    return session_pool


"""~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~"""


class OracleConnection(object):
    """
    SQL Connection Class

    :return: None
    """

    def __init__(self, min_connections: int = 2, max_connections: int = 5):
        self._username: Optional[str] = None
        self._password: Optional[str] = None
        self._hostname: Optional[str] = None
        self._port: Optional[int] = None
        self._database: Optional[str] = None
        self._sid: Optional[str] = None
        self._session_pool: Optional[ConnectionPool] = None

        self.min_connections = min_connections
        self.max_connections = max_connections

        self._retry_count = 0
        self.retry_limit = 50
        self.retry_error_codes = ['ORA-01033', 'DPY-6005', 'DPY-4011', 'ORA-08103']

    @retry
    def _connect(self) -> None:
        """
        Connect to the warehouse

        :return: None
        """

        sid_or_service = self._database if self._database else self._sid

        self._session_pool = _connect_warehouse(self._username, self._password, self._hostname, self._port,
                                                sid_or_service, self.min_connections, self.max_connections)

    def set_user(self, credentials_dict: dict) -> None:
        """
        Set the user credentials and connect

        :param credentials_dict: dictionary of connection details
        :return: None
        """

        if not ([credentials_dict['Service'] or credentials_dict['SID']]):
            raise ValueError("Either Service or SID must be provided")

        self._username: Optional[str] = credentials_dict['UserName']
        self._password: Optional[str] = credentials_dict['Password']
        self._hostname: Optional[str] = credentials_dict['Host']
        self._port: Optional[int] = int(credentials_dict['Port'])
        self._database: Optional[str] = credentials_dict['Service'] if 'Service' in credentials_dict else None
        self._sid: Optional[str] = credentials_dict['SID'] if 'SID' in credentials_dict else None

        self._connect()

    def close_connection(self) -> None:
        """
        Close the connection

        :return: None
        """

        self._session_pool.close()

    @retry
    def execute(self, query: str) -> None:
        """
        Execute the query

        :param query: query
        :return: None
        """

        connection = self._session_pool.acquire()
        cursor = connection.cursor()
        cursor.execute(query)
        connection.commit()
        self._session_pool.release(connection)

    @retry
    def safe_execute(self, query: str, packed_values: dict) -> None:
        """
        Execute the query without SQL Injection possibility, to be used with external facing projects.

        :param query: query
        :param packed_values: dictionary of values
        :return: None
        """

        connection = self._session_pool.acquire()
        cursor = connection.cursor()
        cursor.execute(query, packed_values)
        connection.commit()
        self._session_pool.release(connection)

    @retry
    def execute_multiple(self, queries: list[list[str, dict]]) -> None:
        """
        Execute multiple queries

        :param queries: list of queries
        :return: None
        """

        connection = self._session_pool.acquire()
        cursor = connection.cursor()
        for item in queries:
            query = item[0]
            packed_values = item[1]
            if packed_values:
                cursor.execute(query, packed_values)
            else:
                cursor.execute(query)
        connection.commit()
        self._session_pool.release(connection)

    @retry
    def execute_many(self, query: str, dictionary: list[dict]) -> None:
        """
        Execute many queries

        :param query: query
        :param dictionary: dictionary of values
        :return: None
        """

        connection = self._session_pool.acquire()
        cursor = connection.cursor()
        cursor.executemany(query, dictionary)
        connection.commit()
        self._session_pool.release(connection)

    @retry
    def fetch_data(self, query: str, packed_data=None) -> list:
        """
        Fetch the data from the query

        :param query: query
        :param packed_data: packed data
        :return: rows
        """

        connection = self._session_pool.acquire()
        cursor = connection.cursor()
        if packed_data:
            cursor.execute(query, packed_data)
        else:
            cursor.execute(query)
        rows = cursor.fetchall()
        self._session_pool.release(connection)
        return rows

    @retry
    def remove_matching_data(self, dfObj: pd.DataFrame, outputTableName: str, match_cols: list) -> None:
        """
        Remove matching data from the warehouse

        :param dfObj: DataFrame
        :param outputTableName: output table name
        :param match_cols: list of columns
        :return: None
        """

        df = dfObj[match_cols]
        param_list = []
        for column in match_cols:
            param_list.append(f"{column} = :{column}")
        if len(param_list) > 1:
            params = ' AND '.join(param_list)
        else:
            params = param_list[0]

        main_dict = df.to_dict('records')
        query = f"""DELETE FROM {outputTableName} WHERE {params}"""
        self.execute_many(query, main_dict)

    @retry
    def export_DF_to_warehouse(self, dfObj: pd.DataFrame, outputTableName: str, columns: list, remove_nan=False) -> None:
        """
        Export the DataFrame to the warehouse

        :param dfObj: DataFrame
        :param outputTableName: output table name
        :param columns: list of columns
        :param remove_nan: remove NaN values
        :return: None
        """

        col = ', '.join(columns)
        bindList = []
        for column in columns:
            bindList.append(':' + column)
        bind = ', '.join(bindList)

        if remove_nan:
            dfObj = dfObj.replace({np.nan: None})
        main_dict = dfObj.to_dict('records')

        query = """INSERT INTO {} ({}) VALUES ({})""".format(outputTableName, col, bind)
        self.execute_many(query, main_dict)

    @retry
    def truncate_table(self, tableName: str) -> None:
        """
        Truncate the table

        :param tableName: table name
        :return: None
        """

        truncateQuery = """TRUNCATE TABLE {}""".format(tableName)
        self.execute(truncateQuery)

    @retry
    def empty_table(self, tableName: str) -> None:
        """
        Empty the table

        :param tableName: table name
        :return: None
        """

        deleteQuery = """DELETE FROM {}""".format(tableName)
        self.execute(deleteQuery)

    def __del__(self) -> None:
        """
        Destructor

        :return: None
        """

        self._session_pool.close()


class AsyncOracleConnection(object):
    """
    SQL Connection Class

    :return: None
    """

    def __init__(self, min_connections: int = 2, max_connections: int = 5):
        self._db_service: str = "Oracle"
        self._username: Optional[str] = None
        self._password: Optional[str] = None
        self._hostname: Optional[str] = None
        self._port: Optional[int] = None
        self._database: Optional[str] = None
        self._sid: Optional[str] = None
        self._session_pool: Optional[AsyncConnectionPool] = None

        self.min_connections = min_connections
        self.max_connections = max_connections

        self._retry_count = 0
        self.retry_limit = 50
        self.retry_error_codes = ['ORA-01033', 'DPY-6005', 'DPY-4011', 'ORA-08103']

    @async_retry
    async def _connect(self) -> None:
        """
        Connect to the warehouse

        :return: None
        """

        sid_or_service = self._database if self._database else self._sid

        self._session_pool = await _async_connect_warehouse(self._username, self._password, self._hostname, self._port,
                                                            sid_or_service, self.min_connections, self.max_connections)

    async def set_user(self, credentials_dict: dict) -> None:
        """
        Set the user credentials and connect

        :param credentials_dict: dictionary of connection details
        :return: None
        """

        if not ([credentials_dict['Service'] or credentials_dict['SID']]):
            raise ValueError("Either Service or SID must be provided")

        self._username: Optional[str] = credentials_dict['UserName']
        self._password: Optional[str] = credentials_dict['Password']
        self._hostname: Optional[str] = credentials_dict['Host']
        self._port: Optional[int] = int(credentials_dict['Port'])
        self._database: Optional[str] = credentials_dict['Service'] if 'Service' in credentials_dict else None
        self._sid: Optional[str] = credentials_dict['SID'] if 'SID' in credentials_dict else None

        await self._connect()

    async def close_connection(self) -> None:
        """
        Close the connection

        :return: None
        """

        await self._session_pool.close()

    @async_retry
    async def execute(self, query: str) -> None:
        """
        Execute the query

        :param query: query
        :return: None
        """

        async with self._session_pool.acquire() as connection:
            with connection.cursor() as cursor:
                await cursor.execute(query)
                await connection.commit()

    @async_retry
    async def safe_execute(self, query: str, packed_values: dict) -> None:
        """
        Execute the query without SQL Injection possibility, to be used with external facing projects.

        :param query: query
        :param packed_values: dictionary of values
        :return: None
        """

        async with self._session_pool.acquire() as connection:
            with connection.cursor() as cursor:
                await cursor.execute(query, packed_values)
                await connection.commit()

    @async_retry
    async def execute_multiple(self, queries: list[list[str, dict]]) -> None:
        """
        Execute multiple queries

        :param queries: list of queries
        :return: None
        """

        async with self._session_pool.acquire() as connection:
            with connection.cursor() as cursor:
                for item in queries:
                    query = item[0]
                    packed_values = item[1]
                    if packed_values:
                        await cursor.execute(query, packed_values)
                    else:
                        await cursor.execute(query)
                await connection.commit()

    @async_retry
    async def execute_many(self, query: str, dictionary: list[dict]) -> None:
        """
        Execute many queries

        :param query: query
        :param dictionary: dictionary of values
        :return: None
        """

        async with self._session_pool.acquire() as connection:
            with connection.cursor() as cursor:
                await cursor.executemany(query, dictionary)
                await connection.commit()

    @async_retry
    async def fetch_data(self, query: str, packed_data=None) -> list:
        """
        Fetch the data from the query

        :param query: query
        :param packed_data: packed data
        :return: rows
        """

        async with self._session_pool.acquire() as connection:
            with connection.cursor() as cursor:
                if packed_data:
                    await cursor.execute(query, packed_data)
                else:
                    await cursor.execute(query)
                rows = await cursor.fetchall()
        return rows

    @async_retry
    async def remove_matching_data(self, dfObj: pd.DataFrame, outputTableName: str, match_cols: list) -> None:
        """
        Remove matching data from the warehouse

        :param dfObj: DataFrame
        :param outputTableName: output table name
        :param match_cols: list of columns
        :return: None
        """

        df = dfObj[match_cols]
        param_list = []
        for column in match_cols:
            param_list.append(f"{column} = :{column}")
        if len(param_list) > 1:
            params = ' AND '.join(param_list)
        else:
            params = param_list[0]

        main_dict = df.to_dict('records')
        query = f"""DELETE FROM {outputTableName} WHERE {params}"""
        await self.execute_many(query, main_dict)

    @async_retry
    async def export_DF_to_warehouse(self, dfObj: pd.DataFrame, outputTableName: str, columns: list, remove_nan=False) -> None:
        """
        Export the DataFrame to the warehouse

        :param dfObj: DataFrame
        :param outputTableName: output table name
        :param columns: list of columns
        :param remove_nan: remove NaN values
        :return: None
        """

        col = ', '.join(columns)
        bindList = []
        for column in columns:
            bindList.append(':' + column)
        bind = ', '.join(bindList)

        if remove_nan:
            dfObj = dfObj.replace({np.nan: None})
        main_dict = dfObj.to_dict('records')

        query = """INSERT INTO {} ({}) VALUES ({})""".format(outputTableName, col, bind)
        await self.execute_many(query, main_dict)

    @async_retry
    async def truncate_table(self, tableName: str) -> None:
        """
        Truncate the table

        :param tableName: table name
        :return: None
        """

        truncateQuery = """TRUNCATE TABLE {}""".format(tableName)
        await self.execute(truncateQuery)

    @async_retry
    async def empty_table(self, tableName: str) -> None:
        """
        Empty the table

        :param tableName: table name
        :return: None
        """

        deleteQuery = """DELETE FROM {}""".format(tableName)
        await self.execute(deleteQuery)



from pathlib import Path
from typing import Optional

from get_gecko_driver import GetGeckoDriver
from selenium import webdriver
from selenium.webdriver.chrome.service import Service
from webdriver_manager.chrome import ChromeDriverManager

from wcp_library.selenium._selenium_driver import SeleniumDriver


class ChromeSeleniumHelper(SeleniumDriver):
    def __init__(self, headless: bool = False, download_path: Optional[Path] = None):
        super().__init__(headless, download_path)

        opt = webdriver.ChromeOptions()
        opt.add_argument("--start-maximized")
        if headless:
            opt.add_argument('headless')

        opt.page_load_strategy = 'eager'

        experimental_options_dict = {"download.prompt_for_download": False,
                                        "download.directory_upgrade": True,
                                        "safebrowsing.enabled": True}
        if download_path:
            experimental_options_dict["download.default_directory"] = str(download_path)

        opt.add_experimental_option("prefs", experimental_options_dict)
        opt.timeouts = {'implicit': 5000}

        self.driver = webdriver.Chrome(service=Service(ChromeDriverManager().install()), options=opt)


class FirefoxSeleniumHelper(SeleniumDriver):
    def __init__(self, headless: bool = False, download_path: Optional[Path] = None):
        super().__init__(headless, download_path)

        opt = webdriver.FirefoxOptions()
        opt.add_argument("--start-maximized")
        if headless:
            opt.add_argument('--headless')

        opt.page_load_strategy = 'eager'

        if download_path:
            opt.set_preference("browser.download.folderList", 2)
            opt.set_preference("browser.download.dir", str(download_path))
            opt.set_preference("browser.helperApps.neverAsk.saveToDisk", "application/octet-stream")

        get_driver = GetGeckoDriver()
        get_driver.install()
        self.driver = webdriver.Firefox(options=opt)


import logging
import sys

from wcp_library import application_path


def create_log(file_level: int, console_level: int, iterations: int, project_name: str, mode: str = "w",
               format: str = "%(asctime)s:%(levelname)s:%(module)s:%(filename)s:%(lineno)d:%(message)s"):
    """
    Create log file.

    Log levels: CRITICAL, ERROR, WARNING, INFO, DEBUG, NOTSET

    format help: https://docs.python.org/3/library/logging.html#logrecord-attributes

    :param file_level: Logging level to output to log file.
    :param console_level: Logging level to output to console.
    :param iterations: Number of log files to keep.
    :param project_name: Name of the project. (Used as the log file name)
    :param mode: Mode to open the log file. (Default: "w")
    :param format: Log Format (Default: "%(asctime)s:%(levelname)s:%(module)s:%(filename)s:%(lineno)d:%(message)s")
    :return:
    """

    possible_iterative_filenames = [(application_path / (project_name + ".log"))] + [application_path / (project_name + f"_{i + 1}.log") for i in range(iterations)]

    if iterations == 0:
        (application_path / (project_name + ".log")).unlink(missing_ok=True)
    else:
        last_file = possible_iterative_filenames[-1]
        last_file.unlink(missing_ok=True)

        for i in range(iterations, 0, -1):
            if possible_iterative_filenames[i - 1].exists():
                possible_iterative_filenames[i - 1].rename(possible_iterative_filenames[i])

    logging.basicConfig(
        filename=(application_path / (project_name + ".log")),
        level=file_level,
        format=format,
        filemode=mode
    )

    MIN_LEVEL = console_level
    stdout_hdlr = logging.StreamHandler(sys.stdout)
    stderr_hdlr = logging.StreamHandler(sys.stderr)
    stdout_hdlr.setLevel(MIN_LEVEL)
    stderr_hdlr.setLevel(max(MIN_LEVEL, logging.WARNING))

    rootLogger = logging.getLogger()
    rootLogger.addHandler(stdout_hdlr)
    rootLogger.addHandler(stderr_hdlr)
    logger = logging.getLogger(__name__)
    logger.setLevel(console_level)



import logging
import re
import stat
from pathlib import Path
from typing import Optional

import paramiko

logger = logging.getLogger(__name__)


class SFTP:
    def __init__(self, host: Optional[str]=None, port: Optional[int]=21, password_vault_dict: Optional[dict]=None):
        self.host: str = host if not password_vault_dict else password_vault_dict['Host']
        self.port: int = port if not password_vault_dict else password_vault_dict['Port']
        self._username: Optional[str] = None if not password_vault_dict else password_vault_dict['UserName']
        self._password: Optional[str] = None if not password_vault_dict else password_vault_dict['Password']

        self.ssh = paramiko.SSHClient()
        # AutoAddPolicy automatically adds the hostname and new host key to the local HostKeys object
        self.ssh.set_missing_host_key_policy(paramiko.AutoAddPolicy())

        if not (self._username and self._password):
            self.sftp_connection: Optional[paramiko.SFTP] = None
        else:
            self.ssh.connect(self.host, self.port, self._username, self._password)
            self.sftp_connection = self.ssh.open_sftp()

    def login(self, username: str, password: str) -> None:
        """
        Login to the SFTP server

        :param username:
        :param password:
        :return:
        """

        logger.debug(f"Logging into {self.host} with username {username}")
        self.ssh.connect(self.host, self.port, username, password)
        self.sftp_connection = self.ssh.open_sftp()

    def download(self, remote_file: Path, local_file: Path) -> None:
        """
        Download a file from the SFTP server

        :param remote_file:
        :param local_file:
        :return:
        """

        local_file.parent.mkdir(parents=True, exist_ok=True)

        logger.debug(f"Downloading {remote_file} to {local_file}")
        self.sftp_connection.get(str(remote_file), local_file)

    def download_files(self, local_dir: Path, regex_pattern: str='*') -> None:
        """
        Download files from the SFTP server matching the regex pattern

        :param local_dir:
        :param regex_pattern:
        :return:
        """

        local_dir.mkdir(parents=True, exist_ok=True)

        logger.debug(f"Downloading files from FTP server matching {regex_pattern} to {local_dir}")
        files = self.list_files()
        for file in files:
            if re.match(regex_pattern, file.name):
                logger.debug(f"Downloading {file} to {local_dir / file.name}")
                self.sftp_connection.get(str(file), local_dir / file.name)

    def list_files(self) -> list[Path]:
        """
        List files on the SFTP server

        Returns a list of file paths

        :return:
        """

        logger.debug(f"Listing files on {self.sftp_connection.getcwd()}")
        return [Path(x) for x in self.sftp_connection.listdir() if stat.S_ISREG(self.sftp_connection.lstat(x).st_mode)]

    def list_dirs(self) -> list[Path]:
        """
        List directories on the SFTP server

        Returns a list of directory paths

        :return:
        """

        logger.debug(f"Listing directories on {self.sftp_connection.getcwd()}")
        return [Path(x) for x in self.sftp_connection.listdir() if stat.S_ISDIR(self.sftp_connection.lstat(x).st_mode)]

    def change_dir(self, remote_dir: Path) -> None:
        """
        Change the directory on the SFTP server

        :param remote_dir:
        :return:
        """

        logger.debug(f"Changing FTP directory to {remote_dir}")
        self.sftp_connection.chdir(str(remote_dir))

    def upload(self, local_file: Path, remote_file: Path) -> None:
        """
        Upload a file to the SFTP server

        :param local_file:
        :param remote_file:
        :return:
        """

        logger.debug(f"Uploading {local_file} to {remote_file}")
        self.sftp_connection.put(local_file, str(remote_file))

    def close(self):
        """
        Close the SFTP connection

        :return:
        """

        logger.debug("Closing FTP connection")
        self.sftp_connection.close()
        self.sftp_connection = None


import ftplib
import logging
import re
from pathlib import Path
from typing import Optional

import ftputil

logger = logging.getLogger(__name__)


class FTP:
    def __init__(self, host: Optional[str]=None, port: Optional[int]=21, password_vault_dict: Optional[dict]=None):
        self.host: str = host if not password_vault_dict else password_vault_dict['Host']
        self.port: int = port if not password_vault_dict else password_vault_dict['Port']
        self._username: Optional[str] = None if not password_vault_dict else password_vault_dict['UserName']
        self._password: Optional[str] = None if not password_vault_dict else password_vault_dict['Password']

        self._ftp_factory: ftputil.session.session_factory = ftputil.session.session_factory(base_class=ftplib.FTP_TLS,
                                                                                             port=self.port,
                                                                                             encrypt_data_channel=True,
                                                                                             encoding="UTF-8")
        self.ftp_connection: Optional[ftputil.FTPHost] = None if not (self._username and self._password) \
            else ftputil.FTPHost(self.host, self._username, self._password, session_factory=self._ftp_factory)

    def login(self, username: str, password: str) -> None:
        """
        Login to the FTP server

        :param username:
        :param password:
        :return:
        """

        logger.debug(f"Logging into {self.host} with username {username}")
        self.host = ftputil.FTPHost(self.host, username, password, session_factory=self._ftp_factory)

    def download(self, remote_file: Path, local_file: Path) -> None:
        """
        Download a file from the FTP server

        :param remote_file:
        :param local_file:
        :return:
        """

        local_file.parent.mkdir(parents=True, exist_ok=True)

        logger.debug(f"Downloading {remote_file} to {local_file}")
        self.ftp_connection.download(remote_file, local_file)

    def download_files(self, local_dir: Path, regex_pattern: str='*') -> None:
        """
        Download files from the FTP server matching the regex pattern

        :param local_dir:
        :param regex_pattern:
        :return:
        """

        local_dir.mkdir(parents=True, exist_ok=True)

        logger.debug(f"Downloading files from FTP server matching {regex_pattern} to {local_dir}")
        files = self.list_files()
        for file in files:
            if re.match(regex_pattern, file.name):
                logger.debug(f"Downloading {file} to {local_dir / file.name}")
                self.ftp_connection.download(file, local_dir / file.name)

    def list_files(self) -> list[Path]:
        """
        List files on the FTP server

        Returns a list of file paths

        :return:
        """

        logger.debug(f"Listing files on {self.ftp_connection.curdir}")
        return [Path(x) for x in self.ftp_connection.listdir(self.ftp_connection.curdir) if self.ftp_connection.path.isfile(x)]

    def list_dirs(self) -> list[Path]:
        """
        List directories on the FTP server

        Returns a list of directory paths

        :return:
        """

        logger.debug(f"Listing directories on {self.ftp_connection.curdir}")
        return [Path(x) for x in self.ftp_connection.listdir(self.ftp_connection.curdir) if self.ftp_connection.path.isdir(x)]

    def change_dir(self, remote_dir: Path) -> None:
        """
        Change the directory on the FTP server

        :param remote_dir:
        :return:
        """

        logger.debug(f"Changing FTP directory to {remote_dir}")
        self.ftp_connection.chdir(remote_dir)

    def upload(self, local_file: Path, remote_file: Path) -> None:
        """
        Upload a file to the FTP server

        :param local_file:
        :param remote_file:
        :return:
        """

        logger.debug(f"Uploading {local_file} to {remote_file}")
        self.ftp_connection.upload(local_file, remote_file)

    def close(self) -> None:
        """
        Close the FTP connection

        :return:
        """

        logger.debug(f"Closing FTP connection")
        self.ftp_connection.close()
        self.ftp_connection = None


import smtplib
from email import encoders
from email.mime.base import MIMEBase
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
from email.utils import formatdate
from pathlib import Path


def send_email(sender: str, recipients: list, subject: str, message: str=None) -> None:
    """
    Function to send an email

    :param sender:
    :param recipients:
    :param subject:
    :param message:
    :return:
    """

    msg = MIMEMultipart()
    msg['From'] = sender
    msg['To'] = ", ".join(recipients)
    msg['Date'] = formatdate(localtime=True)
    msg['Subject'] = subject
    msg.attach(MIMEText(message))

    smtpServer = 'mail.wcap.ca'
    server = smtplib.SMTP(smtpServer, 25)
    server.ehlo()
    server.sendmail(sender, recipients, msg.as_string())
    server.quit()


def email_reporting(subject: str, message: str) -> None:
    """
    Function to email the reporting team from the Python email

    :param subject:
    :param message:
    :return:
    """

    msg = MIMEMultipart()
    msg['From'] = "Python@wcap.ca"
    msg['To'] = "Reporting@wcap.ca"
    msg['Date'] = formatdate(localtime=True)
    msg['Subject'] = subject
    msg.attach(MIMEText(message))

    smtpServer = 'mail.wcap.ca'
    server = smtplib.SMTP(smtpServer, 25)
    server.ehlo()
    server.sendmail("Python@wcap.ca", 'Reporting@wcap.ca', msg.as_string())
    server.quit()


def email_with_attachments(sender: str, recipients: list, subject: str, message: str=None, attachments: list[Path]=None) -> None:
    """
    Function to send an email with attachments

    File paths must be passed as a list of Path (pathlib.Path) objects

    :param sender:
    :param recipients:
    :param subject:
    :param message:
    :param attachments:
    :return:
    """

    msg = MIMEMultipart()
    msg['From'] = sender
    msg['To'] = ", ".join(recipients)
    msg['Date'] = formatdate(localtime=True)
    msg['Subject'] = subject
    msg.attach(MIMEText(message))

    for attachment in attachments:
        part = MIMEBase('application', "octet-stream")
        with open(attachment, 'rb') as file:
            part.set_payload(file.read())
        encoders.encode_base64(part)
        part.add_header('Content-Disposition', 'attachment; filename={}'.format(attachment.name))
        msg.attach(part)

    smtpServer = 'mail.wcap.ca'
    server = smtplib.SMTP(smtpServer, 25)
    server.ehlo()
    server.sendmail(sender, recipients, msg.as_string())
    server.quit()